/***************************************************************************
                          DbgSpawn.cpp  -  description
                             -------------------
    begin                : Fri Mar 2 2001
    copyright            : (C) 2001 by Dmitri Dmitrienko
                         : (C) 2002, 2006 NuSphere Corp.
    www                  : http://dd.cron.ru
                         : http://www.nusphere.com/
    license              : This source file is subject to version 1.01 of 
                           the License,  that is bundled with this package 
                           in the file LICENSE, and is available at through 
                           the world-wide-web at http://dd.cron.ru/license
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#if HAVE_UNISTD_H
#include <unistd.h>
#endif

#ifdef HAVE_OSTREAM
#include <iostream>
#endif

#include <signal.h>
#include "dbgbase_globals.h"


pid_t spawn_process(const char *filename, char ** env,
	pipe_t *hpipeINPUT, pipe_t *hpipeOUTPUT, pipe_t *hpipeERROR) {

    int pid = -1, ok;
    pipe_t	in_fd[2] = {INVALID_PIPE,INVALID_PIPE}, 
			out_fd[2] = {INVALID_PIPE,INVALID_PIPE}, 
			err_fd[2] = {INVALID_PIPE,INVALID_PIPE};

	if (hpipeINPUT) *hpipeINPUT = INVALID_PIPE;
	if (hpipeOUTPUT) *hpipeOUTPUT = INVALID_PIPE;
	if (hpipeERROR) *hpipeERROR =  INVALID_PIPE;

    ok = (pipe(in_fd) >= 0 &&
    	  pipe(out_fd) >= 0 &&
    	  pipe(err_fd) >= 0);
    if (!ok) {
    	int err=errno;
    	CLOSEPIPE(in_fd[0]);
    	CLOSEPIPE(in_fd[1]);
    	CLOSEPIPE(out_fd[0]);
    	CLOSEPIPE(out_fd[1]);
    	CLOSEPIPE(err_fd[0]);
    	CLOSEPIPE(err_fd[1]);
    	errno = err;
    	return -1;
    }
    ok = ((pid=fork()) >= 0);
    if (!ok) {
    	int err=errno;
    	CLOSEPIPE(in_fd[0]);
    	CLOSEPIPE(in_fd[1]);
    	CLOSEPIPE(out_fd[0]);
    	CLOSEPIPE(out_fd[1]);
    	CLOSEPIPE(err_fd[0]);
    	CLOSEPIPE(err_fd[1]);
    	errno = err;
    	return -1;
    }

    if (!pid) {
// child
		CLOSEPIPE(out_fd[0]);
		dup2(out_fd[1], STDOUT_FILENO);
		CLOSEPIPE(out_fd[1]);
		CLOSEPIPE(in_fd[1]);
		dup2(in_fd[0], STDIN_FILENO);
		CLOSEPIPE(in_fd[0]);
		CLOSEPIPE(err_fd[0]);
		dup2(err_fd[1],STDERR_FILENO);
		CLOSEPIPE(err_fd[1]);
		signal(SIGCHLD, SIG_DFL);
		signal(SIGPIPE, SIG_IGN);
		signal(SIGINT, SIG_IGN);
		execve(filename, NULL, env);
		std::cerr << "\nfailed to spawn \"" << filename << "\", errno=" << errno << ", strerror=" << strerror(errno) << "\n";
		exit(1); // if execve() failed
    }
// parent
    CLOSEPIPE(out_fd[1]);
    CLOSEPIPE(in_fd[0]);
    CLOSEPIPE(err_fd[1]);
	
	
	
    if (hpipeINPUT) {
	*hpipeINPUT = in_fd[1];
    } else {
	CLOSEPIPE(in_fd[1]);
    }
    if (hpipeOUTPUT) {
	*hpipeOUTPUT = out_fd[0];
    } else {
	CLOSEPIPE(out_fd[0]);
    }
    if (hpipeERROR) {
	*hpipeERROR = err_fd[0];
    } else {
	CLOSEPIPE(err_fd[0]);
    }
    return pid;
}

